/* vRouter: simreader
 *
 *  Purpose: Read and execute a simulation, based on stream input
 *
 *  Changes:
 *   0.02 - Initial release
 *
 */


#include<iostream.h>
#include<string>

#include "ipaddress.h"
#include "router.h"
#include "packet.h"
#include "flow.h"
#include "network.h"
#include "simreader.h"
#include "simulator.h"
#include "parse.h"

#include<iostream>
#include <vector>
#include <sys/time.h>

/*
 * Constructor
 */
Simreader::Simreader(Network *n)
{
  net = n;
  sim = new Simulator(net);

  /* seed the random number generator */
  struct timeval tv;
  struct timezone tz;
  gettimeofday(&tv, &tz);
  srand(tv.tv_usec % 100);
}

/*
 * Destructor
 */
Simreader::~Simreader()
{
  if(sim)
    delete sim;
}

/* 
 * Parse and run a simulation from the given imput stream
 */

void Simreader::read(istream &s)
{
  parse(s);
}

/*
 * Parse a run simulation command.  Run the simulation.
 */
void Simreader::parse_runsim(istream &in)
{
  int sliceWidth, sliceCount;
  sliceWidth = getInt(in);
  sliceCount = getInt(in);

  sim->runSim(sliceWidth, sliceCount);
}

void Simreader::parse(istream &in)
{

  string token = "";
  token = getString(in);

  while(token != "end-simulation")
    {
      if(token == "flow")
	{
	  Flow f = parse_flow(in);
	  sim->addFlow(f);
	}
      else if(token == "run")
	{
	  parse_runsim(in);
	}
      else
	{
	  cerr << "Simreader: Parse error on " << token << endl;
	  return;
	}
      token = getString(in);
    }

}


Flow Simreader::parse_flow(istream &in)
{
  string srcRouter, src, dst;
  int pktsize, pktrate, starttime, duration;

  srcRouter = getString(in);
  src = getString(in);
  dst = getString(in);
  pktsize = getInt(in);
  pktrate = getInt(in);
  starttime= getInt(in);
  duration = getInt(in);

  Flow f(net->getRouterByName(srcRouter), IPAddress(src), IPAddress(dst), pktsize, pktrate, starttime, duration);
  
  return f;
  
}























